"""
Notolog Editor
Open-source markdown editor developed in Python.

File Details:
- Purpose: Part of the base module.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from ..settings import Settings
from ..app_config import AppConfig
from ..lexemes.lexemes import Lexemes
